# -*- coding: utf-8 -*-
# Pretty ~ Useful ~ Python

from typing import Any
from typing import Optional

def timestamp(ts: Optional[float] = ...) -> str: ...
def environ_dict(): ...
def linked_tmp_dir(
    suffix: Optional[Any] = ...,
    prefix: Optional[Any] = ...,
    dir: Optional[Any] = ...,
    mkdirs: Any = ...,
    lndirs: Any = ...,
    lnfiles: Any = ...,
) -> None: ...
def prinfo(obj: Any) -> None: ...
def pyfilepath(split: bool = ...): ...
def time_funk(funk: Any, *args: Any, **kwargs: Any): ...
def cmp_funks(f1: Any, f2: Any, runs: int = ..., *args: Any, **kwargs: Any): ...
